class CollectionsController < ApplicationController
  before_action :set_collection, only: [:show, :update, :destroy, :like, :add_learning_object, :remove_learning_object, :change_privacy]
  before_action :authenticate_user!, only: [:update, :destroy, :like, :add_learning_object, :remove_learning_object, :change_privacy]

  # GET /collections
  # GET /collections.json
  def index
    respond_to do |format|

      # for HTML page, returns institutional collections
      format.html do
        context = Collections::InstitutionsContext.new(institution_repository.all)
        @collections = collection_repository.all context
      end

      # returns all collection of logged user as JSON for UI actions
      if user_signed_in?
        format.json do
          @collections = current_user.collections
        end
      end
    end
  end

  # GET /collections/1
  # GET /collections/1.json
  def show
    check_collection_privacy! @collection

    @user = @collection.owner
    @own = user_signed_in? ? @collection.owner?(current_user) : false
  end

  # POST /collections/1/like
  def like
    collection_repository.like @collection
  end

  # POST /collections
  # POST /collections.json
  def create
    authorize_create_collection!
    @collection = Collection.new(collection_params)
    @collection.owner = current_user if collection_params[:owner].blank?

    respond_to do |format|
      if collection_repository.create @collection
        format.html { redirect_to :back, notice: 'Coleção criada com sucesso.' }
      else
        format.html { render :new }
      end
    end
  end

  # PATCH/PUT /collections/1
  # PATCH/PUT /collections/1.json
  def update
    respond_to do |format|
      if collection_repository.update(collection_params)
        format.html { redirect_to @collection, notice: 'Coleção atualizada com sucesso.' }
      else
        format.html { render :edit }
      end
    end
  end

  # DELETE /collections/1
  # DELETE /collections/1.json
  def destroy
    collection_repository.destroy @collection

    respond_to do |format|
      format.html { redirect_to me_users_path, notice: 'Coleção excluida com sucesso.' }
    end
  end

  # POST /collections/1/learning_object/43
  def add_learning_object
    learning_object = learning_object_repository.find params[:learning_object_id]
    @collection.add learning_object

    if request.xhr? && collection_repository.save_learning_objects(@collection)
      render json: {status: true}
    end
  end

  # DELETE /collections/1/learning_object/43
  def remove_learning_object
    learning_object = learning_object_repository.find params[:learning_object_id]
    @collection.remove learning_object
    collection_repository.save_learning_objects @collection

    if request.xhr?
      render json: {status: true}
    end
  end

  # change collection privacy
  def change_privacy
    collection_repository.change_privacy(@collection, params[:privacy])

    if request.xhr?
      render json: {status: true}
    end
  end

  private

  def authorize_create_collection!
  end

  def check_collection_privacy!(collection)
    if collection.privacy == "private"
      redirect_to :root, notice: 'Está é uma coleção privada.' unless collection.owner?(current_user)
    end
  end

  def set_collection
    @collection = collection_repository.find params[:id]
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def collection_params
    params.require(:collection).permit(:name, :description, :owner,learning_objects: [])
  end

end
